@@ -193,7 +193,7 @@ main (gint argc, gchar *argv[])
 	/* Translators: this messagge will apper immediately after the
 	 * usage string - Usage: COMMAND <THIS_MESSAGE>
 	 */
-	context = g_option_context_new (_("- start the user guides data miner"));
+	context = g_option_context_new (_("— start the user guides data miner"));
 
 	g_option_context_add_main_entries (context, entries, NULL);
 	g_option_context_parse (context, &argc, &argv, &error);
